#ifndef __JABYENGINE_CD_INTERNAL_HPP__
#define __JABYENGINE_CD_INTERNAL_HPP__
#include "cd_types.hpp"

namespace JabyEngine {
    namespace CD {
        namespace internal {
            extern CD_IO::Interrupt::Type last_interrupt;
            extern State                  current_state;

            static CD_IO::Interrupt::Type read_last_interrupt() {
                return const_cast<volatile CD_IO::Interrupt::Type&>(last_interrupt);
            }

            static State read_current_state() {
                return const_cast<volatile State&>(current_state);
            }

            struct Command {
                static void wait_until(CD_IO::Interrupt::Type irq) {
                    while(read_last_interrupt() != irq);
                }

                template<typename...ARGS>
                static void send(CD_IO::CommandFifo_v& cmd_fifo, CD_IO::ParameterFifo_v& parameter_fifo, CD_IO::Command::Desc cmd, ARGS...args) {
                    while(CD_IO::IndexStatus.is_set(CD_IO::IndexStatus_t::IsTransmissionBusy));

                    ((parameter_fifo = args),...);
                    cmd_fifo = cmd.id;
                }

                template<typename T, typename...ARGS>
                static void send(CD_IO::Command::Desc cmd, ARGS...args) {
                    send(T::CommandFifo, T::ParameterFifo, cmd, args...);
                }

                template<typename...ARGS>
                static void send_wait(CD_IO::CommandFifo_v& cmd_fifo, CD_IO::ParameterFifo_v& parameter_fifo, CD_IO::Command::Desc cmd, ARGS...args) {
                    send(cmd_fifo, parameter_fifo, cmd, args...);
                    wait_until(cmd.complete_irq);
                }

                template<typename T, typename...ARGS>
                static void send_wait(CD_IO::Command::Desc cmd, ARGS...args) {
                    send_wait(T::CommandFifo, T::ParameterFifo, cmd, args...);
                }
            };

            void read_file(FileInfo file_info, const SectorBufferAllocator& buffer_allocator);
            bool continue_reading();
        }
    }
}
#endif //!__JABYENGINE_CD_INTERNAL_HPP__