#include "../../../internal-include/CD/cd_internal.hpp"
#include <PSX/File/Processor/cd_file_processor.hpp>
#include <stdio.h>

namespace JabyEngine {
    void CDFileProcessor :: start_cur_job() {
        using CD::internal::FileInfo;
        using CD::internal::SectorBufferAllocator;
            const auto configurate_for = [this](const CDFile& file) -> SectorBufferAllocator {
                static const auto circular_buffer_callback = [](void* ctx) -> CD_IO::DataSector* {
                    CDFileProcessor &self = *reinterpret_cast<CDFileProcessor*>(ctx);
                    return self.circular_buffer.allocate();
                };
                static const auto direct_copy_callback     = [](void* ctx) -> CD_IO::DataSector* {
                    auto** data_sector = reinterpret_cast<CD_IO::DataSector**>(ctx);
                    auto*  cur_sector  = *data_sector;

                    data_sector += sizeof(CD_IO::DataSector);
                    return cur_sector;
                };

                switch(file.type) {
                    case CDFileType::SimpleTIM:
                        printf("CDFileProcessor: SimpleTIM not supported yet\n");
                        return SectorBufferAllocator::invalid();

                    case CDFileType::CopyTo:
                        this->dst_area = file.payload.copy_to.dst;
                        return SectorBufferAllocator::create(&this->dst_area, direct_copy_callback);

                    default:
                        return SectorBufferAllocator::invalid();
                }            
            };

        const auto& cur_job = *this->jobs.files;
        const auto& cur_lba = this->lba[cur_job.rel_lba_idx];
        const auto  cfg     = configurate_for(cur_job);

        CD::internal::read_file(FileInfo::from(cur_lba), cfg);

        printf(">>> CD needs to load LBA: %i -> %i\n", cur_lba.lba, cur_lba.size_words);
    }

    void CDFileProcessor :: reading_state(const CDFile& file) {
        switch(file.type) {
            case CDFileType::SimpleTIM:
                printf("CDFileProcessor: SimpleTIM not supported yet\n");
                break;

            case CDFileType::CopyTo:
            default:
                break;
        }
    }

    void CDFileProcessor :: done_state(const CDFile& file) {
        switch(file.type) {
            case CDFileType::SimpleTIM:
                printf("CDFileProcessor: SimpleTIM not supported yet\n");
                break;

            case CDFileType::CopyTo:
            default:
                break;
        }
    }

    void CDFileProcessor :: setup(const volatile AutoLBAEntry* lba, JobArray jobs, uint32_t* tmp_area) {
        this->lba      = const_cast<const AutoLBAEntry*>(lba);
        this->jobs     = jobs;

        // Setsup the circular buffer and determines where to place the temp area in
        const_cast<uint32_t*&>(this->tmp_area) = reinterpret_cast<uint32_t*>(this->circular_buffer.setup(reinterpret_cast<CD_IO::DataSector*>(tmp_area), 5));
        CDFileProcessor::start_cur_job();
    }

    Progress CDFileProcessor :: process() {
        const auto& cur_job = *this->jobs.files;

        switch(CD::internal::read_current_state()) {
            case CD::internal::State::Done:
                // We are done now!
                // The user decides if he wants the next value
                CDFileProcessor::done_state(cur_job);
                return Progress::Done;

            case CD::internal::State::BufferFull:
                // We have to process data and unpause the CD drive
                CDFileProcessor::reading_state(cur_job);
                return CD::internal::continue_reading() ? CDFileProcessor::process() : Progress::Error;

            case CD::internal::State::Reading:
                // Do we have data? Use it!
                CDFileProcessor::reading_state(cur_job);
                return Progress::InProgress;

            case CD::internal::State::Error:
            default:
                // Error for real!
                return Progress::Error;
        }
    }
}